import argparse
import sys

from .harvest import *
from .transform import transform


class CustomHelpFormatter(argparse.HelpFormatter):
    """
    Custom formatting class for argparse
    """

    def _format_action(self, action):
        if type(action) == argparse._SubParsersAction:
            # inject new class variable for subcommand formatting
            subactions = action._get_subactions()
            invocations = [self._format_action_invocation(a) for a in subactions]
            self._subcommand_max_length = max(len(i) for i in invocations)

        if type(action) == argparse._SubParsersAction._ChoicesPseudoAction:
            # format subcommand help line
            subcommand = self._format_action_invocation(action)  # type: str
            width = self._subcommand_max_length
            help_text = ""
            if action.help:
                help_text = self._expand_help(action)
            return "  {:{width}} -  {}\n".format(subcommand, help_text, width=width)

        elif type(action) == argparse._SubParsersAction:
            # process subcommand help section
            msg = '\n'
            for subaction in action._get_subactions():
                msg += self._format_action(subaction)
            return msg
        else:
            return super(CustomHelpFormatter, self)._format_action(action)


def argument_parser():
    """Application level args"""

    arg_parser = argparse.ArgumentParser(
        description='manatus - Metadata aggregation and transformation utility',
        usage="[-p | --profile <profile> ] | [-h] [-v] [--test] | <command> [-h] | <subcommand>",
        add_help=True,
        formatter_class=CustomHelpFormatter)
    subcommand_parsers = arg_parser.add_subparsers(help='subcommands', dest='subcommand')
    subcommand_parsers.required = False
    subcommand_parsers.add_parser('status', help='show status')

    # generic config parser
    # adds options to harvest & transform subcommands
    config_parser = argparse.ArgumentParser(add_help=False)
    config_parser.add_argument('-n', '--new', action='store_true', help='add a new config entry')
    config_parser.add_argument('-l', '--list', action='store_true', help='list config entries')
    config_parser.add_argument('-i', '--interactive', action='store_true',
                               help="select entry from config interactively")
    config_parser.add_argument('-s', '--select', help="run action on a specific config entry", metavar='config_entry')

    # harvest subcommand & args
    harvest_parser = subcommand_parsers.add_parser('harvest', help='manatus harvest interactions',
                                                   parents=[config_parser])
    harvest_parser.add_argument('-r', '--run', action='store_true', help='run harvest for all config entries')

    # transform subcommand & args
    transformation_parser = subcommand_parsers.add_parser('transform', help='manatus transformation interactions',
                                                          parents=[config_parser])
    transformation_parser.add_argument('-r', '--run', action='store_true',
                                       help='run transformation for all config entries')
    transformation_parser.add_argument('--to_console', action='store_true',
                                       help="print records to console, don't write to disk")

    arg_parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', help='verbose mode')
    arg_parser.add_argument('-p', '--profile', dest='profile', help='select configuration profile to use')
    arg_parser.add_argument('--test', dest='test', action='store_true', help='run module unit tests')

    # custom help message
    arg_parser._positionals.title = "commands"

    # hack to show help when no arguments supplied
    if len(sys.argv) == 1:
        arg_parser.print_help()
        sys.exit(0)

    return arg_parser


def config_list(config_parser):
    """
    Simple printing of config options

    :param config_parser:
    :return:

    """
    for section in config_parser.sections():
        print(f'\n[{section}]')
        for k, v in config_parser[section].items():
            print(f'{k}: {v}')
    sys.exit(0)


def interactive_run(manatus_config, config_parser, subcommand, profile, *args, **kwargs):
    """
    Allows interactive selection of organization to harvest/transform

    :param profile:
    :param manatus_config:
    :param config_parser:
    :param subcommand:
    :param args:
    :param kwargs:
    :return:

    """
    i = 0
    print(f'Please select the number for an organization to {subcommand} from the list below:\n')
    for section in config_parser.sections():
        i = i + 1
        print(f'{i}. {section}')
    try:
        selection = int(input('\nOrganization >>> '))
    except ValueError:
        print("\nSorry I didn't quite get that.")
        print('Please only input the number corresponding to the organization.')
        selection = int(input('\nOrganization >>> '))
    if subcommand == 'harvest':
        write_path = args[0]
        verbosity = kwargs['verbosity']
        harvest(config_parser[config_parser.sections()[selection - 1]],
                [k for k in config_parser.sections()][selection - 1], write_path, verbosity=verbosity)
    elif subcommand == 'transform':
        verbosity = kwargs['verbosity']
        to_console = kwargs['to_console']
        transform(manatus_config, config_parser[config_parser.sections()[selection - 1]],
                  [k for k in config_parser.sections()][selection - 1], profile, verbosity=verbosity,
                  to_console=to_console)


def new_config_entry(config_file, options_list, config_fp):
    """
    Function to add entries to config file

    :param config_file:
    :param options_list:
    :param config_fp:
    :return:

    """
    with open(config_fp, 'w') as f:
        org_key = str(input('What is the organization key: >>> '))
        config_file.add_section(org_key)
        for option in options_list:
            setting = str(input(f'Enter a value/s for {option}: >>> '))
            config_file.set(org_key, option, setting)
        config_file.write(f)


empty_manatus_cfg = r'''# Please see manatus configuration documentation: https://manatus.readthedocs.io"

[DEFAULT]
InFilePath = 
OutFilePath = 
OutFilePrefix = 
CustomMapPath = 
CustomMapTestPath = 
CustomMapTestName = 
LogPath = 
LogLevel =
Provider = 
CustomMapPath =  

[named_profile]
# For profile specific options
'''

empty_harvest_cfg = r'''# Please see manatus configuration documentation: https://manatus.readthedocs.io"

[manatus_provider_key]
oaiendpoint = 
setlist = 
metadataprefix =                     
'''

empty_scenario_cfg = r'''# Please see manatus configuration documentation: https://manatus.readthedocs.io"

[manatus_provider_key]
scenario = 
map = 
dataprovider = 
intermediateprovider =                    
'''



import datetime
import logging
import os

import sickle
from sickle import models
from sickle.iterator import OAIItemIterator
from sickle.utils import xml_to_dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class SickleRecord(models.Record):

    def __init__(self, record_element, strip_ns=True):
        models.Record.__init__(self, record_element, strip_ns=strip_ns)

    def get_metadata(self):
        try:
            self.metadata = xml_to_dict(
                self.xml.find(
                    './/' + self._oai_namespace + 'metadata'
                ).getchildren()[0], strip_ns=self._strip_ns)
        except AttributeError:
            self.metadata = None


def harvest(org_harvest_info, org_key, write_path, verbosity):
    """
    Manatus harvest function

    :param dict org_harvest_info: Dict of data from ``manatus_harvests.cfg``.
        Includes the key, value pairs :py:data:`oaiendpoint`, :py:data:`setlist`, and :py:data:`metadataprefix`.
        See :doc:`configuration` for more information.
    :param str org_key: Section key in ``manatus_harvests.cfg``. Appended to :py:data:`write_path`
    :param str write_path: File path to write data. Taken from ``manatus.cfg``
    :param int verbosity: Set verbosity

    """
    logger.debug('cli.harvest called')
    # check for dir path
    if not os.path.exists(os.path.join(write_path, org_key)):
        logger.debug(f'Creating path {os.path.join(write_path, org_key)}')
        os.makedirs(os.path.join(write_path, org_key))

    # OAI-PMH endpoint URL from config
    oai = org_harvest_info['OAIEndpoint']

    # metadataPrefix from config
    metadata_prefix = org_harvest_info['MetadataPrefix']

    # iterate through sets to harvest
    for set_spec in org_harvest_info['SetList'].split(', '):
        logger.debug(f'Harvesting {org_key} set {set_spec}')
        if verbosity > 1:
            print(f'Harvesting {org_key} set {set_spec}')

        # Sickle harvester
        harvester = sickle.Sickle(oai, iterator=OAIItemIterator, encoding='utf-8')
        harvester.class_mapping['ListRecords'] = SickleRecord

        records = harvester.ListRecords(set=set_spec, metadataPrefix=metadata_prefix, ignore_deleted=True)

        # write XML
        with open(os.path.join(write_path, org_key, f'{set_spec.replace(":", "_")}_{datetime.date.today()}.xml'), 'w',
                  encoding='utf-8') as fp:
            logger.debug(f'Writing records {fp.name}')
            fp.write('<oai>')
            for record in records:
                fp.write(record.raw)
            fp.write('</oai>')



import json
import os
import unittest
from datetime import date

# from manatus.maps import mods_standard_map
from manatus.cli import transform

test_dir_path = os.path.abspath(os.path.dirname(__file__))


class MainDCTransformTestCase(unittest.TestCase):

    def setUp(self):
        self.config = {'ssdn': {'InFilePath': os.path.join(test_dir_path, 'test_data'),
                                'OutFilePath': os.path.join(test_dir_path, 'test_data'),
                                'Provider': 'Sunshine State Digital Network',
                                'OutFilePrefix': 'SSDN_TMP',
                                'CustomMapPath': test_dir_path}}
        self.data = json.loads(
            '{"@context": "http://api.dp.la/items/context", "aggregatedCHO": "#sourceResource", "dataProvider": \
            "University of South Florida Libraries", "provider": {"@id": "UNDETERMINED", "name": \
            "Sunshine State Digital Network"}, "isShownAt": "oai:USFLDC:SFS0049735_00001", "sourceResource": \
            {"creator": [{"name": "Boucicault, Dion, 1820-1890 ( author )"}], "description": \
            ["Promptbook includes typed pages with extensive notes from the stenographer and minor notes from the author.", \
            "Includes cast list of performers."], "format": ["82 numbered leaves : plans", "24 cm"], "identifier": \
            "oai:USFLDC:SFS0049735_00001", "language": ["English"], "rights": [\
            "The University of South Florida Libraries believes that the Item is in the Public Domain under the laws of the United States, but a determination was not made as to its copyright status under the copyright laws of other countries. The Item may not be in the Public Domain under the laws of other countries."], \
            "subject": [{"name": "Promptbooks -- Manuscripts -- 19th century"}], "title": ["Marriage"], \
            "type": ["text"]}}')

    def tearDown(self):
        os.remove(os.path.join(test_dir_path, 'test_data', f'SSDN_TMP-{date.today()}.jsonl'))

    def test_main_dc_transform(self):
        transformation_info = {'Map': 'dc_standard_map',
                               'DataProvider': 'University of South Florida Libraries',
                               'IntermediateProvider': None,
                               'Scenario': 'SSDNDC'}
        transform(self.config, transformation_info, 'dc', 'ssdn', verbosity=1)
        with open(os.path.join(test_dir_path, 'test_data', f'SSDN_TMP-{date.today()}.jsonl')) as fp:
            test_data = json.load(fp)
        self.assertEqual(test_data, self.data)


class MainQDCTransformTestCase(unittest.TestCase):

    def setUp(self):
        self.config = {'ssdn': {'InFilePath': os.path.join(test_dir_path, 'test_data'),
                                'OutFilePath': os.path.join(test_dir_path, 'test_data'),
                                'Provider': 'Sunshine State Digital Network',
                                'OutFilePrefix': 'SSDN_TMP',
                                'CustomMapPath': test_dir_path}}
        self.data = json.loads(
            '{"@context": "http://api.dp.la/items/context", "aggregatedCHO": "#sourceResource", \
            "dataProvider": "University of Miami Libraries", "provider": {"@id": "UNDETERMINED", "name": \
            "Sunshine State Digital Network"}, "isShownAt": "oai:merrick.library.miami.edu:chc0336/421", \
            "sourceResource": {"creator": [{"name": "Consuegra, Jes\u00fas"}], "date": ["1937-11-24"], \
            "description": ["Sent from Havana to New York"], "format": ["Correspondence", "image/tiff"], \
            "identifier": "oai:merrick.library.miami.edu:chc0336/421", "rights": \
            ["The copyright and related rights status of this material is unknown. For additional information, please visit: http://merrick.library.miami.edu/digitalprojects/copyright.html", \
            "http://rightsstatements.org/vocab/UND/1.0/"], "title": \
            ["Jes\u00fas Consuegra letter to Gerardo Machado, November 24, 1937"], "type": ["Text"], \
            "collection": ["University of Miami. Library. Cuban Heritage Collection", \
            "Gerardo Machado y Morales Papers", "CHC0336"], "extent": ["1 item (1 p.)"]}}')

    def tearDown(self):
        os.remove(os.path.join(test_dir_path, 'test_data', f'SSDN_TMP-{date.today()}.jsonl'))

    def test_main_qdc_transform(self):
        transformation_info = {'Map': 'qdc_standard_map',
                               'DataProvider': 'University of Miami Libraries',
                               'IntermediateProvider': None,
                               'Scenario': 'SSDNQDC'}
        transform(self.config, transformation_info, 'qdc', 'ssdn', verbosity=1)
        with open(os.path.join(test_dir_path, 'test_data', f'SSDN_TMP-{date.today()}.jsonl')) as fp:
            test_data = json.load(fp)
        self.assertEqual(test_data, self.data)


class MainMODSTransformTestCase(unittest.TestCase):

    def setUp(self):
        self.config = {'ssdn': {'InFilePath': os.path.join(test_dir_path, 'test_data'),
                                'OutFilePath': os.path.join(test_dir_path, 'test_data'),
                                'Provider': 'Sunshine State Digital Network',
                                'OutFilePrefix': 'SSDN_TMP',
                                'CustomMapPath': test_dir_path}}
        self.data = json.loads(
            '{"@context": "http://api.dp.la/items/context", "aggregatedCHO": "#sourceResource", "dataProvider": \
            "Florida State University Libraries", "provider": {"@id": "UNDETERMINED", "name": \
            "Sunshine State Digital Network"}, "isShownAt": "oai:fsu.digital.flvc.org:fsu_370933", "sourceResource": \
            {"contributor": [{"name": "Levy Bros."}], "creator": [{"name": "Eppes, N.W."}], "date": {"displayDate": \
            "1894-01-01", "begin": "1894-01-01", "end": "1894-01-01"}, "extent": ["2 pages", "7 x 16 cm"], "format": \
            [{"name": "Money", "@id": "http://id.loc.gov/vocabulary/graphicMaterials/tgm006725"}], "identifier": \
            "oai:fsu.digital.flvc.org:fsu_370933", "language": [{"name": "English", "iso_639_3": "eng"}], "spatial": \
            {"name": "Checks--United States"}, "rights": "http://rightsstatements.org/vocab/InC/1.0/", "subject": \
            [{"@id": "http://id.loc.gov/vocabulary/graphicMaterials/tgm006725", "name": "Money"}, {"@id": \
            "http://id.loc.gov/vocabulary/graphicMaterials/tgm001917", "name": "Checks"}, {"@id": \
            "http://id.loc.gov/vocabulary/graphicMaterials/tgm001479", "name": "Business & finance"}, {"@id": \
            "http://id.loc.gov/authorities/subjects/sh85022834", "name": "Checks"}, {"@id": \
            "http://id.loc.gov/authorities/subjects/sh2008100311", "name": "Checks--United States"}, \
            {"@id": "http://id.loc.gov/authorities/subjects/sh85086790", "name": "Money"}], "title": \
            "Capital City Bank Check for $1.00 to Levy Bros, signed by N. W. Eppes", "type": "text"}}')

    def tearDown(self):
        os.remove(os.path.join(test_dir_path, 'test_data', f'SSDN_TMP-{date.today()}.jsonl'))

    def test_main_mods_transform(self):
        transformation_info = {'Map': 'mods_standard_map',
                               'DataProvider': 'Florida State University Libraries',
                               'IntermediateProvider': None,
                               'Scenario': 'SSDNMODS'}
        transform(self.config, transformation_info, 'mods', 'ssdn', verbosity=1)
        with open(os.path.join(test_dir_path, 'test_data', f'SSDN_TMP-{date.today()}.jsonl')) as fp:
            test_data = json.load(fp)
        self.assertEqual(test_data, self.data)


if __name__ == '__main__':
    unittest.main()



# import unittest


# class ExceptionsTestCase(unittest.TestCase):
# from manatus import exceptions

# def test_something(self):
#     self.assertEqual(True, False)


# if __name__ == '__main__':
#     unittest.main()



"""
Container classes for the resulting DPLA MAPv4 JSON-LD document(s)
"""

import json
import logging
import os
from datetime import date
from json import JSONEncoder
from os.path import exists, join, splitext

from manatus.exceptions import SourceResourceRequiredElementException, RecordGroupFileExtensionError

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Record(object):

    def __init__(self):
        """
        Generic object class. Provides some JSON-like methods
        """
        object.__init__(self)

    def __contains__(self, item):
        if item in self.__dict__.keys():
            return True
        else:
            return False

    def __iter__(self):
        for k in self.__dict__.keys():
            yield k

    def __delitem__(self, key):
        if key in self.__dict__.keys():
            del self.__dict__[key]
        else:
            raise KeyError

    def __getitem__(self, item):
        if item in self.__dict__.keys():
            return self.__dict__[item]
        else:
            raise KeyError

    def __setattr__(self, key, value):
        if value:
            self.__dict__[key] = value

    def __setitem__(self, key, value):
        if value:
            self.__dict__[key] = value

    def __str__(self):
        try:
            return f'{self.__class__.__name__}, {self.__dict__["identifier"]}'
        except KeyError:
            return f'{self.__repr__()}'

    def dumps(self, indent=None):
        """
        Return object's JSON representation. Useful for debugging

        :param indent: padding for pretty printing
        :type indent: integer or None
        :return: record JSON
        :rtype: str
        :meta hide-value:

        """
        return json.dumps(self.__dict__, indent=indent)

    @property
    def data(self):
        """Property for raw access to record's data at ``self.__dict__``"""
        return self.__dict__

    def keys(self):
        """JSON key iterator"""
        for k in self.__dict__.keys():
            yield k


class DPLARecord(Record):

    def __init__(self, record=None):
        """
        DPLA MAPv4 record class. Serves as the JSON-LD wrapper for :py:class:`SourceResource`. Includes some default
        attributes for convenience.
        """
        Record.__init__(self)
        if record:
            try:
                for k, v in json.loads(record).items():
                    self.__dict__[k] = v
            except TypeError:
                for k, v in record.items():
                    self.__dict__[k] = v
        else:
            self.__dict__['@context'] = "http://api.dp.la/items/context"
            self.aggregatedCHO = "#sourceResource"
            self.preview = ""


class SourceResource(Record):

    def __init__(self):
        """
        DPLA MAPv4 sourceResource record class

        :raises SourceResourceRequiredElementException: if either required elements 'rights' or 'title' are missing

        """
        Record.__init__(self)

    def __setattr__(self, key, value):
        if key == 'rights' and not value:
            raise SourceResourceRequiredElementException(self, 'Rights')
        elif key == 'title' and not value:
            raise SourceResourceRequiredElementException(self, 'Title')
        elif value:
            self.__dict__[key] = value


class RecordGroup(object):

    def __init__(self, records=None):
        """
        List container for :py:class:`scenarios.ManatusRecord` records

        :param list records: List of :py:class:`scenarios.ManatusRecord` records or subclassed records

        """
        object.__init__(self)
        if records:
            self.records = self.records + [DPLARecord(rec) for rec in records]
        else:
            self.records = []

    def __iter__(self):
        for r in self.records:
            yield r

    def __len__(self):
        return len(self.records)

    def append(self, record):
        self.records.append(record)

    def load(self, fp):
        """
        Load record group from file

        :param str fp: File path
        :return: :py:class:`manatus.source_resource.RecordGroup`
        :raises RecordGroupFileExtensionError: if the file does not have a `.json` or `.jsonl` extension
        :raises FileNotFoundError: is the specified file doesn't exist
        """
        if exists(fp):
            with open(fp) as f:
                if splitext(fp)[-1] == '.jsonl':
                    for line in f:
                        self.append(DPLARecord(line))
                elif splitext(fp)[-1] == '.json':
                    recs = json.load(f)
                    for rec in recs:
                        self.append(DPLARecord(rec))
                else:
                    raise RecordGroupFileExtensionError(fp)
            return self
        else:
            raise FileNotFoundError

    def write_json(self, fp, prefix=None, pretty_print=False):
        """

        :param str fp:
        :param str prefix:
        :param bool pretty_print:
        :return:
        """
        if not exists(fp):
            os.mkdir(fp)
        if prefix:
            f = f'{prefix}-{date.today()}'
        else:
            f = f'{date.today()}'
        if exists(join(fp, f'{f}.json')):
            with open(join(fp, f'{f}.json'), 'r', encoding='utf-8') as json_in:
                data = json.load(json_in)
                for record in data:
                    self.records.append(record)
            with open(join(fp, f'{f}.json'), 'w', encoding='utf-8') as json_out:
                if pretty_print:
                    json.dump(self.records, json_out, indent=2, cls=DPLARecordEncoder)
                else:
                    json.dump(self.records, json_out, cls=DPLARecordEncoder)
        else:
            with open(join(fp, f'{f}.json'), 'w', encoding='utf-8') as json_out:
                if pretty_print:
                    json.dump(self.records, json_out, indent=2, cls=DPLARecordEncoder)
                else:
                    json.dump(self.records, json_out, cls=DPLARecordEncoder)

    def write_jsonl(self, fp, prefix=None):
        """

        :param str fp:
        :param str prefix:
        :return:
        """
        if not exists(fp):
            os.mkdir(fp)
        if prefix:
            f = f'{prefix}-{date.today()}'
        else:
            f = f'{date.today()}'
        with open(join(fp, f'{f}.jsonl'), 'a', encoding='utf-8', newline='\n') as json_out:
            for rec in self.records:
                json_out.write(json.dumps(rec, cls=DPLARecordEncoder) + '\n')

    def print(self, indent=None):
        for rec in self.records:
            print(json.dumps(rec.data, indent=indent, cls=DPLARecordEncoder))


class DPLARecordEncoder(JSONEncoder):

    def default(self, o):
        return o.__dict__


def dedupe_record_group():
    pass



import os
import unittest

test_dir_path = os.path.abspath(os.path.dirname(__file__))


class XMLScenarioTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import XMLScenario
        self.scenario = XMLScenario(os.path.join(test_dir_path, 'test_data/DCdebugSmall.xml'))

    def test_deleted_records(self):
        self.assertEqual(len(self.scenario), 3)


class ManatusRecordTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import XMLScenario, XMLRecord
        self.scenario = XMLScenario(os.path.join(test_dir_path, 'test_data/DCdebugSmall.xml'))
        self.record = XMLRecord(self.scenario.records[0])

    def test_manatus_record_harvest_id(self):
        self.assertEqual(self.record.harvest_id,
                         "oai:lib.fsu.edu.fiu:oai:uofm.library.fiu:oai:dpsobek:FI07050832_00001")


class DCRecordTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import SSDNDC
        self.scenario = SSDNDC(os.path.join(test_dir_path, 'test_data/DCdebugSmall.xml'))
        self.record = self.scenario.records[0]

    def test_DC_record_contributor(self):
        self.assertEqual(self.record.contributor, ['Buckethead'])

    def test_DC_record_creator(self):
        self.assertEqual(self.record.creator, ['Lewis Carroll'])

    def test_DC_record_date(self):
        self.assertEqual(self.record.date, ['1974'])

    def test_DC_record_description(self):
        self.assertEqual(self.record.description, ['1 postcard, postally unused',
                                                   'caption: "Alligator Joe watching the Young Alligators Hatch."'])

    def test_DC_record_format(self):
        self.assertEqual(self.record.format, ["Embroidery"])

    def test_DC_record_identifier(self):
        self.assertEqual(self.record.identifier, ['FI07050832',
                                                  'http://dpanther.fiu.edu/dpService/dpPurlService/purl/FI07050832/00001'])

    def test_DC_record_language(self):
        self.assertEqual(self.record.language, ['English'])

    def test_DC_record_place(self):
        self.assertEqual(self.record.place, ["Moon"])

    def test_DC_record_publisher(self):
        self.assertEqual(self.record.publisher,
                         ['Miami, Florida: J.N. Chamberlain', 'Miami, Florida: J.N. Chamberlain'])

    def test_DC_record_rights(self):
        self.assertEqual(self.record.rights, ["Rights 4A"])

    def test_DC_record_subject(self):
        self.assertEqual(self.record.subject, ['Alligators--Florida--Everglades.'])

    def test_DC_record_title(self):
        self.assertEqual(self.record.title, ["Alligator Joe watching the young alligators hatch"])

    def test_DC_record_type(self):
        self.assertEqual(self.record.type, ["Text"])


class QDCRecordTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import SSDNQDC
        self.scenario = SSDNQDC(os.path.join(test_dir_path, 'test_data/QDCdebugSmall.xml'))
        self.record = self.scenario.records[0]

    def test_QDC_record_abstract(self):
        self.assertEqual(self.record.abstract, ["Test 000"])

    def test_QDC_record_alternative(self):
        self.assertEqual(self.record.alternative, ["covfefe"])

    def test_QDC_record_contributor(self):
        self.assertEqual(self.record.contributor, ['Thee Oh Sees'])

    def test_QDC_record_creator(self):
        self.assertEqual(self.record.creator, ['Gilpin, Vincent'])

    def test_QDC_record_date(self):
        self.assertEqual(self.record.date, ["1933-09-03"])

    def test_QDC_record_description(self):
        self.assertEqual(self.record.description, ['Test 001'])

    def test_QDC_record_extent(self):
        self.assertEqual(self.record.extent, ["1 letter"])

    def test_QDC_record_format(self):
        self.assertEqual(self.record.format, ["image/tiff"])

    def test_QDC_record_identifier(self):
        self.assertEqual(self.record.identifier, ['http://merrick.library.miami.edu/cdm/ref/collection/asm0447/id/31'])

    def test_QDC_record_isPartOf(self):
        self.assertEqual(self.record.is_part_of, ['University of Miami. Library. Special Collections',
                                                  'Alan Crockwell Collection',
                                                  'ASM0447'])

    def test_QDC_record_language(self):
        self.assertEqual(self.record.language, ['eng'])

    def test_QDC_record_medium(self):
        self.assertEqual(self.record.medium, ['found objects', 'realia'])

    def test_QDC_record_place(self):
        self.assertEqual(self.record.place, ["Excelsior (Minn.)"])

    def test_QDC_record_publisher(self):
        self.assertEqual(self.record.publisher, ['Arista Records'])

    def test_QDC_record_requires(self):
        self.assertEqual(self.record.requires, ['noharvest'])

    def test_QDC_record_rights(self):
        self.assertEqual(self.record.rights,
                         ["Rights 4A", "Copyright Undetermined http://rightsstatements.org/page/UND/1.0/"])

    def test_QDC_record_subject(self):
        self.assertEqual(self.record.subject,
                         ['Gilpin, Vincent',
                          'Munroe, Patty',
                          'Munroe, Ralph, 1851-1933',
                          'Letters'])

    def test_QDC_record_title(self):
        self.assertEqual(self.record.title, ["Vincent Gilpin letter to Patty Munroe, September 3, 1933"])

    def test_QDC_record_type(self):
        self.assertEqual(self.record.type, ["Tax return from a VIP"])


class MODSRecordTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import SSDNMODS
        self.scenario = SSDNMODS(os.path.join(test_dir_path, 'test_data/MODSdebugSmall.xml'))
        self.record = self.scenario.records[0]

    def test_MODS_record_alternative(self):
        self.assertEqual(self.record.alternative, ['ALT: Fraternity fundraiser for injured student'])

    def test_MODS_record_collection_title(self):
        self.assertEqual(self.record.collection.title, 'General Jean-Jacques-Germain Pelet-Clozeau Papers')

    def test_MODS_record_collection_location(self):
        self.assertEqual(self.record.collection.location,
                         'Special Collections & Archives, Florida State University, Tallahassee, Florida')

    def test_MODS_record_collection_url(self):
        self.assertEqual(self.record.collection.url, 'https://archives.lib.fsu.edu/repositories/10/resources/531')

    def test_MODS_record_contributor(self):
        self.assertEqual(self.record.contributor, [{'name': 'Winkler, '}])

    def test_MODS_record_creator(self):
        self.assertEqual(self.record.creator, [{'name': 'Mauldin, Bob'}])

    def test_MODS_record_date(self):
        self.assertEqual(self.record.date, {"begin": "1935", "end": "1969", "displayDate": "1935 - 1969"})

    def test_MODS_record_description(self):
        self.assertEqual(self.record.description, ["Test 00"])

    def test_MODS_record_extent(self):
        self.assertEqual(self.record.extent, ["8 x 10 in."])

    def test_MODS_record_format(self):
        self.assertEqual(self.record.format, [{"name": "Photographs"}])

    def test_MODS_record_identifier(self):
        self.assertEqual(self.record.identifier, "http://purl.flvc.org/fsu/fd/FSUspcn329b")

    def test_MODS_record_language(self):
        self.assertEqual(self.record.language, [{"name": "English", "iso_639_3": "eng"}])

    def test_MODS_record_place(self):
        self.assertEqual(self.record.place, {"name": "Narnia"})

    def test_MODS_record_publisher(self):
        self.assertEqual(self.record.publisher, ["Tay Tay Frankie"])

    def test_MODS_record_rights(self):
        self.assertEqual(self.record.rights, 'http://rightsstatements.org/vocab/NoC-US/1.0/')

    def test_MODS_record_subject(self):
        self.assertEqual(self.record.subject,
                         [{'name': 'Students'}, {'name': 'Greek life'}, {'name': 'Fraternities and Sororities'},
                          {'name': 'Narnia'}])

    def test_MODS_record_title(self):
        self.assertEqual(self.record.title, 'Fraternity fundraiser for injured student')

    def test_MODS_record_type(self):
        self.assertEqual(self.record.type, "still image")


# def suite():
#     xml_scenario_suite = unittest.makeSuite(XMLScenarioTestCase, 'test')
#     manatus_record_suite = unittest.makeSuite(ManatusRecordTestCase, 'test')
#     dc_record_suite = unittest.makeSuite(DCRecordTestCase, 'test')
#     qdc_record_suite = unittest.makeSuite(QDCRecordTestCase, 'test')
#     mods_record_suite = unittest.makeSuite(MODSRecordTestCase, 'test')
#     test_suite = unittest.TestSuite((xml_scenario_suite, manatus_record_suite, dc_record_suite, qdc_record_suite, mods_record_suite))
#     return test_suite


if __name__ == '__main__':
    unittest.main()



import os
import unittest

test_dir_path = os.path.abspath(os.path.dirname(__file__))


class DCStandardMapTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import SSDNDC, dc_standard_map
        self.scenario = SSDNDC(os.path.join(test_dir_path, 'test_data/DCdebugSmall.xml'))
        for sr, _ in dc_standard_map(self.scenario[0]):
            self.sr = sr

    def test_dc_standard_map(self):
        self.assertEqual(self.sr.data, {'contributor': [{'name': 'Buckethead'}],
                                        'creator': [{'name': 'Lewis Carroll'}],
                                        'date': ['1974'],
                                        'description': ['1 postcard, postally unused',
                                                        'caption: "Alligator Joe watching the Young Alligators Hatch."'],
                                        'format': ['Embroidery'],
                                        'identifier': 'oai:lib.fsu.edu.fiu:oai:uofm.library.fiu:oai:dpsobek:FI07050832_00001',
                                        'language': ['English'],
                                        'spatial': [{'name': 'Moon'}],
                                        'publisher': ['Miami, Florida: J.N. Chamberlain',
                                                      'Miami, Florida: J.N. Chamberlain'],
                                        'rights': ['Rights 4A'],
                                        'subject': [{'name': 'Alligators--Florida--Everglades.'}],
                                        'title': ['Alligator Joe watching the young alligators hatch'],
                                        'type': ['Text']})


class QDCStandardMapTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import SSDNQDC, qdc_standard_map
        self.scenario = SSDNQDC(os.path.join(test_dir_path, 'test_data/QDCdebugSmall.xml'))
        for sr, _ in qdc_standard_map(self.scenario[0]):
            self.sr = sr

    def test_qdc_standard_map(self):
        self.assertEqual(self.sr.data, {'abstract': ['Test 000'],
                                        'alternative': ['covfefe'],
                                        'collection': ['University of Miami. Library. Special Collections',
                                                       'Alan Crockwell Collection',
                                                       'ASM0447'],
                                        'contributor': [{'name': 'Thee Oh Sees'}],
                                        'creator': [{'name': 'Gilpin, Vincent'}],
                                        'date': ['1933-09-03'],
                                        'description': ['Test 001'],
                                        'extent': ['1 letter'],
                                        'format': ['image/tiff'],
                                        'identifier': 'oai:lib.fsu.edu.umiami:oai:uofm.library.umiami:oai:merrick.library.miami.edu:asm0447/31',
                                        'language': ['eng'],
                                        'spatial': [{'name': 'Excelsior (Minn.)'}],
                                        'publisher': ['Arista Records'],
                                        'rights': ['Rights 4A',
                                                   'Copyright Undetermined http://rightsstatements.org/page/UND/1.0/'],
                                        'subject': [{'name': 'Gilpin, Vincent'},
                                                    {'name': 'Munroe, Patty'},
                                                    {'name': 'Munroe, Ralph, 1851-1933'},
                                                    {'name': 'Letters'}],
                                        'title': ['Vincent Gilpin letter to Patty Munroe, September 3, 1933'],
                                        'type': ['Tax return from a VIP']})


class MODSStandardMapTestCase(unittest.TestCase):

    def setUp(self):
        from manatus import SSDNMODS, mods_standard_map
        self.scenario = SSDNMODS(os.path.join(test_dir_path, 'test_data/MODSdebugSmall.xml'))
        for sr, _ in mods_standard_map(self.scenario[0]):
            self.sr = sr

    def test_mods_standard_map(self):
        self.assertEqual(self.sr.data, {'alternative': ['ALT: Fraternity fundraiser for injured student'],
                                        'collection': 'General Jean-Jacques-Germain Pelet-Clozeau Papers',
                                        'contributor': [{'name': 'Winkler, '}],
                                        'creator': [{'name': 'Mauldin, Bob'}],
                                        'date': {'begin': '1935', 'displayDate': '1935 - 1969', 'end': '1969'},
                                        'description': ['Test 00'],
                                        'extent': ['8 x 10 in.'],
                                        'format': [{'name': 'Photographs'}],
                                        'identifier': 'oai:lib.fsu.edu.fsu_digital_library:oai:fsu.digital.flvc.org:fsu_24694',
                                        'language': [{'iso_639_3': 'eng', 'name': 'English'}],
                                        'spatial': {'name': 'Narnia'},
                                        'publisher': ['Tay Tay Frankie'],
                                        'rights': 'http://rightsstatements.org/vocab/NoC-US/1.0/',
                                        'subject': [{'name': 'Students'},
                                                    {'name': 'Greek life'},
                                                    {'name': 'Fraternities and Sororities'},
                                                    {'name': 'Narnia'}],
                                        'title': 'Fraternity fundraiser for injured student',
                                        'type': 'still image'})


if __name__ == '__main__':
    unittest.main()



import logging
import os
import sys

import manatus
import manatus.maps

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def build(custom_map_function, data, org, provider):
    """apply transformation map to data iterable"""
    logger.debug('cli.build called')
    records = manatus.RecordGroup()

    logger.debug(f'Mapping data with {custom_map_function}')
    mapped_data = map(custom_map_function, data)

    logger.debug(f'Mapped data with {custom_map_function}')
    for mapped_rec in mapped_data:
        # map generators can return None if record is marked to be skipped or errors
        if mapped_rec:
            for sr, tn, *args in mapped_rec:

                logger.info(f"Building {sr.data['identifier']}")
                dpla = manatus.DPLARecord()
                if args:
                    dpla.dataProvider = args[0]
                else:
                    dpla.dataProvider = org.data_provider

                if args:
                    dpla.intermediateProvider = args[1]
                else:
                    dpla.intermediateProvider = org.intermediate_provider

                dpla.provider = {'@id': 'UNDETERMINED', 'name': provider}
                dpla.isShownAt = sr.data['identifier']
                dpla.preview = tn
                dpla.sourceResource = sr.data

                logger.debug(f"Built record {sr.data['identifier']}")
                records.append(dpla.data)

        else:
            continue

    return records


def transform(manatus_config, org_transformation_info, org_key, profile, verbosity, to_console=False):
    """
    Manatus transformation function

    :param dict manatus_config: Data from ``manatus.cfg``. See :doc:`configuration` for more information.
    :param dict org_transformation_info: Data from ``manatus_scenarios.cfg``. Includes the key, value pairs
        :py:data:`scenario`, :py:data:`map`, :py:data:`dataprovider`, and *(optional)*
        :py:data:`intermediatedataprovider`. See :doc:`configuration` for more information.
    :param str org_key: Section key in ``manatus_scenarios.cfg``. Used to locate source data for transformation.
    :param str profile: Profile invoked during run
    :param int verbosity: Set verbosity
    :param bool to_console: If set to ``True``, transformed data is written to the console rather than the file system.

    """
    logger.debug('cli.transform called')
    IN_PATH = os.path.abspath(manatus_config[profile]['InFilePath'])
    OUT_PATH = os.path.abspath(manatus_config[profile]['OutFilePath'])
    prefix = manatus_config[profile]['OutFilePrefix']
    provider = manatus_config[profile]['Provider']

    ### IMPORTING CUSTOM MAPS
    custom_map_path = os.path.abspath(manatus_config[profile]['CustomMapPath'])
    sys.path.append(custom_map_path)

    # import config key, value pairs into DataProvider slot attrs
    o = manatus.DataProvider()
    o.key = org_key
    o.map = org_transformation_info['Map']
    o.data_provider = org_transformation_info['DataProvider']
    try:
        o.intermediate_provider = org_transformation_info['IntermediateProvider']
    except KeyError:
        o.intermediate_provider = None

    ###############################################################################
    # These six lines take a string and use it to search the supplied module      #
    # for a callable function with that name.                                     #
    #   custom_map_module can only be imported (and then searched) if             #
    #       1. the module location is added to sys.path (as CustomMapPath is)     #
    #       2. the module name does not collide with another module coming before #
    #           it in the MRO (i.e. name it something unique)                     #
    ###############################################################################

    # use config scenario value to search for manatus.scenarios class
    o.scenario = getattr(manatus, org_transformation_info['Scenario'])
    # use config map value to search for callable module & function with that name
    try:
        logger.debug(f'Trying to find custom map module {o.map}')
        custom_map_module = __import__(o.map)
        custom_map_function = getattr(custom_map_module, o.map)
    except ModuleNotFoundError:
        # if custom lookup fails, fall back to manatus default maps
        custom_map_function = getattr(manatus.maps, o.map)
        logger.info(f'Custom map module {o.map} not found. Using default manatus map')

    # check scenario subclassing
    # XMLScenario subclasses read data from disk
    if issubclass(o.scenario, manatus.XMLScenario):
        for f in os.listdir(os.path.join(IN_PATH, o.key)):

            logger.info(f'Transforming {o.key} data {f}')
            if verbosity > 1:
                print(f'Transforming {o.key} data {f}')
            # parse file using scenario and get records as iterable list

            logger.debug(f'Loading data {f} with {o.scenario}')
            data = o.scenario(os.path.join(IN_PATH, o.key, f))

            logger.debug(f'Loaded data {f} with {o.scenario}')
            records = build(custom_map_function, data, o, provider)

    # APIScenario subclasses need to make queries and read data from responses
    elif issubclass(o.scenario, manatus.APIScenario):

        logger.info(f'Transforming {o.key} data API')
        if verbosity > 1:
            print(f'Transforming {o.key} data from API')

        logger.debug(f'Loading API data with {o.scenario}')
        data = o.scenario(o.key)

        logger.debug(f'Loaded API data with {o.scenario}')
        records = build(custom_map_function, data, o, provider)

    if to_console:
        logger.debug('Printing records')
        records.print()

    else:
        logger.debug(f'Writing records to {OUT_PATH}')
        records.write_jsonl(OUT_PATH, prefix=prefix)
